﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Imaging;

namespace howto_adjust_opacity
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The loaded image.
        private Bitmap OriginalImage = null;

        // The adjusted image.
        private Bitmap AdjustedImage = null;

        // The selected opacity.
        private float OpacityValue = 0.75f;

        // Load a picture.
        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                OriginalImage = new Bitmap(ofdImage.FileName);
                ShowImage();
                picSample.Visible = true;
                mnuFileSave.Enabled = true;
            }
        }

        // Save the adjusted image.
        private void mnuFileSave_Click(object sender, EventArgs e)
        {
            if (sfdImage.ShowDialog() == DialogResult.OK)
            {
                AdjustedImage.Save(sfdImage.FileName, ImageFormat.Png);
            }
        }

        // Draw the loaded image with the selected opacity.
        private void ShowImage()
        {
            // Make sure we have an input picture.
            if (OriginalImage == null) return;

            // Create a sample image.
            // Make a background image.
            Bitmap sample_bm =
                MakeCheckerboard(
                    OriginalImage.Width,
                    OriginalImage.Height, 64);

            // Draw the adjusted image over the checkerboard.
            AdjustedImage = SetOpacity(OriginalImage, OpacityValue);
            using (Graphics gr = Graphics.FromImage(sample_bm))
            {
                gr.DrawImage(AdjustedImage, 0, 0);
            }

            picSample.Image = sample_bm;
        }

        // Make a checkerboard bitmap.
        private Bitmap MakeCheckerboard(int width, int height, int size)
        {
            int num_rows = height / size + 1;
            int num_cols = width / size + 1;
            Bitmap bm = new Bitmap(width, height);
            using (Graphics gr = Graphics.FromImage(bm))
            {
                for (int r = 0; r < num_rows; r++)
                {
                    for (int c = 0; c < num_cols; c++)
                    {
                        Rectangle rect = new Rectangle(
                            c * size, r * size, size, size);
                        if ((r + c) % 2 == 0)
                            gr.FillRectangle(Brushes.Blue, rect);
                        else
                            gr.FillRectangle(Brushes.Yellow, rect);
                    }
                }
            }
            return bm;
        }

        // Set the image's opacity.
        private Bitmap SetOpacity(Bitmap input_bm, float opacity)
        {
            // Make the new bitmap.
            Bitmap output_bm = new Bitmap(
                input_bm.Width, input_bm.Height);

            // Make an associated Graphics object.
            using (Graphics gr = Graphics.FromImage(output_bm))
            {
                // Make a ColorMatrix with the opacity.
                ColorMatrix color_matrix = new ColorMatrix();
                color_matrix.Matrix33 = opacity;

                // Make the ImageAttributes object.
                ImageAttributes attributes = new ImageAttributes();
                attributes.SetColorMatrix(color_matrix,
                    ColorMatrixFlag.Default, ColorAdjustType.Bitmap);

                // Draw the input bitmap onto the Graphics object.
                Rectangle rect = new Rectangle(0, 0,
                    output_bm.Width, output_bm.Height);

                gr.DrawImage(input_bm, rect,
                    0, 0, input_bm.Width, input_bm.Height,
                    GraphicsUnit.Pixel, attributes);
            }
            return output_bm;
        }

        // Change the opacity.
        private void scrOpacity_Scroll(object sender, ScrollEventArgs e)
        {
            OpacityValue = (scrOpacity.Value / 100f);
            lblOpacity.Text = OpacityValue.ToString("0.00");
            ShowImage();
        }
    }
}
